function [blkMatrix tetradIndex tetradOri topMostLine Color] = genNewTetrad(tetradNumber)
% GENNEWTETRAD  - generate new tetrominoes and return their letter types and
% orientations
%
% Usage:
% [blkMatrix tetradType tetradOri topMostLine] = genNewTetrad(tetradNumber)
%
% tetradNumber: How many tetrad to generate, 1 by default
%
% blkMatrix: the data array of the newly generated tetromino
% tetradIndex: the index of the tetromino
% tetradOri: the orientation of the tetromino
% topMostLine: the first non-zero line of the data array, used to determine
% the initial position of the tetromino
% Color: The color of the tetromino

% Update History
% DATE          UPDATER             CHANGES
% ------        ---------           ------------
% Jul.3, 08     MJ Zhang            Original Code

global Tetrad_O;
global Tetrad_I;
global Tetrad_J;
global Tetrad_L;
global Tetrad_S;
global Tetrad_Z;
global Tetrad_T;

global TetrisTypeIndex;
global TetradColor;

if nargin < 1
    tetradNumber = 1;   % Generate One Tetrad By Default
end

tetradIndex = ceil(rand(1, tetradNumber).*7);  % get the type index

tetrad_type = TetrisTypeIndex(tetradIndex);     % get the letter of type
tetradOri = ceil(rand(1, tetradNumber).*4);   % get the orientations
Color = TetradColor(tetradIndex,:);              % Get the color

% Get the data array
blkMatrix = eval(['Tetrad_' upper(tetrad_type) '(:,:,tetradOri);']);


% Get the first line in the data array that contains at least one
% non-zero component
for topMostLine=1:4
    if find(blkMatrix(topMostLine,:)~=0)
        break;
    end
end