function movableFlag = isBlockMovable(blockX, blockY, blkMatrix, whichSide)
% ISBLOCKMOVABLE    - Decide if a block is movable to a given direction
global Playfield_1P
global MOVING_BLOCK SETTLED_BLOCK REFPOINT_ON_BLOCK
global PF_WIDTH_INT PF_HEIGHT_INT

movableFlag = 1;        % By default, we presume the block is movable

%% For the left side
if strcmpi(whichSide, 'left')   % If it is the 'left side'
    
    for i = 1:4                 % For all the 4 rows of a block
        % Find out the position of the leftmost brick in this row.
        % And convert to the Playfield coordinate.
        leftmost_x = find(blkMatrix(i,:) == MOVING_BLOCK, 1, 'first') + ...
                     blockX - REFPOINT_ON_BLOCK.X;
        if isempty(leftmost_x)      % If no brick in this row
            continue;               % try the next row
        end
        
        if (leftmost_x <= 1)     % If leftmost_x is at the left border
            movableFlag = 0;     % Then the block cannot be moved 
            return;
        end
        % Get the Playfield coordinate of the brick
        leftmost_y = i + blockY - REFPOINT_ON_BLOCK.Y;
        
        % If there is a 'settled' brick to the left of the leftmost brick
        
            
        if (Playfield_1P(leftmost_y, leftmost_x - 1) == SETTLED_BLOCK)
            movableFlag = 0;        % Then the block cannot move
            return
        end
    end
    
    movableFlag = 1;                % If all 4 rows are not blocked
    return

%% For the right side
elseif strcmpi(whichSide, 'right')
    for i = 1:4
        % Find out the position of the rightmost brick in this row.
        % And convert to the Playfield coordinate.
        rightmost_x = find(blkMatrix(i,:) == MOVING_BLOCK, 1, 'last') + ...
                      blockX - REFPOINT_ON_BLOCK.X;
        if isempty(rightmost_x)     % If no brick in this row
            continue;               % try the next row
        end
        
        if (rightmost_x >= PF_WIDTH_INT)     % If rightmost_x is at the right border
            movableFlag = 0;     % Then the block cannot be moved 
            return;
        end
         % Get the Playfield coordinate of the brick
        rightmost_y = i + blockY - REFPOINT_ON_BLOCK.Y;
        
        % If there is a 'settled' brick to the right of the rightmost brick
        if (Playfield_1P(rightmost_y, rightmost_x + 1) == SETTLED_BLOCK)
            movableFlag = 0;        % Then the block cannot move
            return
        end
    end
    movableFlag = 1;                % If all 4 rows are not blocked
    return
 
%% For the bottom line
elseif strcmpi(whichSide, 'bottom')
    for i = 1:4
        % Find out the position of the bottommost brick in each column.
        % And convert to the Playfield coordinate.
        bottommost_y = find(blkMatrix(:,i) == MOVING_BLOCK, 1, 'last')+ ...
                       blockY - REFPOINT_ON_BLOCK.Y;
        if (isempty(bottommost_y))
            continue;
        end
        
        if (bottommost_y >= PF_HEIGHT_INT)     % If bottommost_y is at the bottom
            movableFlag = 0;     % Then the block cannot be moved 
            return;
        end
        % Get the Playfield coordinate of the horizontal position
        bottommost_x = i + blockX - REFPOINT_ON_BLOCK.X;
        
        % If there is a 'settled' brick BELOW the bottommost brick
        if (Playfield_1P(bottommost_y+1, bottommost_x) == SETTLED_BLOCK)
            movableFlag = 0;
            return
        end
    end
    movableFlag = 1;
end