function elapsedTime = displayElapsedTime(timerStart, currentTime)
% Display the elapsed time on the screen
% ** Exclusive for Psychophycists' Edition **
%

% Update History
% -------------------------------------------------------
%  Date         Modifier            Modification
% ------        --------            ------------
% July 9, 2009   M. Zhang           Wrote it
% ------        --------            ------------
% -------------------------------------------------------
global w


global ELAPSED_TIME_XY
global ELAPSED_TIME_FONT
global ELAPSED_TIME_SIZE
global ELAPSED_TIME_COLOR

global TimerDisplay



elapsedTime = currentTime - timerStart;     % Get the elapsed Time in seconds

if ~TimerDisplay  % If the timer display is disabled
    return;
end

elapsedHour = floor(elapsedTime/3600);       % Get the hour

restSecs = mod(elapsedTime,3600);

elapsedMin = floor(restSecs/60);

elapsedSec = floor(mod(restSecs, 60));

% Convert to string
hourString  = num2str(elapsedHour);
minString = num2str(elapsedMin);
secString = num2str(elapsedSec);

% Pad with 0's
if elapsedHour < 10
    hourString = ['0' hourString];
end
if elapsedMin < 10
    minString = ['0' minString];
end
if elapsedSec < 10
    secString = ['0' secString];
end
% hourString = [repmat(['0'], 2-floor(log10(elapsedHour+eps)+1),1),  num2str(elapsedHour)]; % Get the strings for input
% minString = [repmat(['0'],  2-floor(log10(elapsedMin+eps)+1),1), num2str(elapsedMin)]; % Padded with 0's
% secString = [repmat(['0'],  2-floor(log10(elapsedSec+eps)+1),1), num2str(elapsedSec)];

timeString = [hourString, ':', minString, ':', secString];  % Assemble them into the final string

Screen('TextSize', w, ELAPSED_TIME_SIZE);
Screen('TextFont', w, ELAPSED_TIME_FONT);
textrect = Screen('TextBounds', w, '0000:00:00');
Screen('FillRect', w, [0 0 0], textrect);   % Clear the old string
Screen('DrawText', w, timeString, ELAPSED_TIME_XY(1),  ELAPSED_TIME_XY(2), ELAPSED_TIME_COLOR); % Put the new time
