function alignedRect = getAlignedRect(subRect, mainRect, rx, ry, xalign, yalign)
% GETALIGNEDRECT - Auto-compute the rect of a region given its position and
% dimension in a larger rect
% subRect: either a 4-element vector (a rect) or a 2-element vector (size
%           of the rect) [x y],
% mainRect: The larger Rect, must be a 4-element vector
% rx: The x position of the rect. This position is represented by the LEFT BORDER,
%     RIGHT BORDER, or CENTER of the rect, which is given by 'xalign'. rx
%     can be usually a number or a string('center')
% xalign: The alignment mode of x coordinate: can be one of the following:
%           'left', 'right', 'center'; when xalign is neglected, it is
%           defaulted to 'center';
% ry, yalign: ditto

%% Get the size of the subRect
if numel(subRect) == 4
    sizeRect = [subRect(3)-subRect(1), subRect(4)-subRect(2)];
elseif numel(subRect) == 2
    sizeRect = subRect;
else
    error('getAlignedRect(): Wrong subRect.');
end

if ~exist('xalign','var') || isempty(xalign)
    xalign = 'center';
end

if ~exist('yalign','var') || isempty(yalign)
    yalign = 'center';
end

sizeMainRect = [mainRect(3)-mainRect(1), mainRect(4)-mainRect(2)];

if ischar(rx) && strcmp(rx, 'center')
    rx = (sizeMainRect(1) - sizeRect(1))./2;
else
    rx = rx - sizeRect(1)./2;
end

switch xalign
    case 'left'
        rx = rx + sizeRect(1)./2;
    case 'right'
        rx = rx - sizeRect(1)./2;
    otherwise
        rx = rx;    % 'center' or other cases
end


if ischar(ry) && strcmp(ry, 'center')
    ry = (sizeMainRect(2) - sizeRect(2))./2;
else
    ry = ry - sizeRect(2)./2;
end
switch yalign
    case 'top'
        ry = ry + sizeRect(2)./2;
    case 'bottom'
        ry = ry - sizeRect(2)./2;
    otherwise
        ry = ry;    % 'center' or other cases
end
        
alignedRect = [rx ry rx ry] + [0 0 sizeRect];