function [returnEntry returnCode] = showMenu(menuEntries, menuProperties)
% SHOWMENU - Display the menu
global w
global winRect

KbName('UnifyKeyNames');

%% Variable Initialization
nEntries = length(menuEntries); % Get the number of entries
FRAMERECTCONST = [0 0 0 0];    % Weird constant. Have to use this because of PTB inconsistency
menuRect = [];
menuWidth = 0;
menuHeight = 0;
menuPos = [];

menuOffWin = [];

menuCurrEntry = menuProperties.DefaultEntry;
% In case the default Entry is disabled
    while ~menuEntries(menuCurrEntry).Enabled
        menuCurrEntry = wrapEntry(menuCurrEntry + 1, nEntries);
    end


%% Decide the size of the menu box

if strcmp(menuProperties.MenuSize, 'auto')  % Auto-adjust the menusize
    
    % ---- Get the maximum width and height --------
    for i = 1:nEntries
        Screen('TextFont', w, menuEntries(i).Font);
        Screen('TextSize', w, menuEntries(i).Size);
        thisTextBounds = Screen('TextBounds', w, menuEntries(i).Caption, 0, 0); % Get the textBounds
        menuWidth = max(menuWidth, thisTextBounds(3) - thisTextBounds(1));
        menuHeight = menuHeight + thisTextBounds(4) - thisTextBounds(2);
    end
    
    % ----- Count the spacings in ------
    menuWidth = menuWidth + menuProperties.HoriMargin.*2;
    menuHeight = menuHeight + menuProperties.VertMargin.*2 + menuProperties.EntryVertSpacing.*(nEntries-1);
else
    menuWidth = menuProperties.MenuSize(1);
    menuHeight = menuProperties.MenuSize(2);
end

%% Decide the position of the menu box
% ----- Get the Correct Position

menuPos = menuProperties.MenuPosition;
if menuProperties.MenuPosition(1) == -1
    menuPos(1) = (winRect(3) - menuWidth)./2;
end
if menuProperties.MenuPosition(2) == -1
    menuPos(2) = (winRect(4) - menuHeight)./2;
end

%% Get the Rect

menuRect = floor(menuPos([1 2 1 2]) + [0 0 menuWidth menuHeight]);

%% Backup the Background

menuOffWin = Screen('OpenOffScreenWindow', w, 0, menuRect); % Create the offscreen window

% ## On Mac, this CopyWindow must be executed before a Flip, otherwise you
% are gonna copy nothing at all! (probably because the framebuffer is cleared)
Screen('CopyWindow', w, menuOffWin, menuRect, []);    % Save the background

%% Paint the Menu Box

Screen('FillRect', w, menuProperties.MenuBkgColor, menuRect);
Screen('FrameRect', w, menuProperties.MenuFrameColor, menuRect+FRAMERECTCONST);

%% Calculate the Text Positions
thisTextXPos = zeros(1, nEntries);
thisTextYPos = zeros(1, nEntries);
thisTextRect = zeros(nEntries, 4);
for i = 1:nEntries
    Screen('TextFont', w, menuEntries(i).Font);
    Screen('TextSize', w, menuEntries(i).Size);
    % --------- Get the X Position -------------------
    if strcmp(menuEntries(i).Type, 'enum')
        thisTextBounds = Screen('TextBounds', w, [menuEntries(i).Caption num2str(menuEntries(i).Limits(2))], 0, 0);
        thisTextBounds = thisTextBounds + [0 0 menuEntries(i).Spacing 0];
    elseif strcmp(menuEntries(i).Type, 'setkey')
        thisTextBounds = Screen('TextBounds', w, [menuEntries(i).Caption menuProperties.LongestKeyName], 0, 0); % Get the textBounds
        thisTextBounds = thisTextBounds + [0 0 menuEntries(i).Spacing 0];
    else
        thisTextBounds = Screen('TextBounds', w, menuEntries(i).Caption, 0, 0); % Get the textBounds
    end
    if strcmp(menuProperties.EntriesAlignment, 'center')  % Center alignment
        thisTextXPos(i) = menuPos(1) + horiCenterTextPos(thisTextBounds, menuRect);
    elseif strcmp(menuProperties.EntriesAlignment, 'left')    % Left alignment
        thisTextXPos(i) = menuPos(1) + menuProperties.HoriMargin;
    end
     % --------- Get the Y Position -------------------
    if i == 1
        thisTextYPos(i) = menuPos(2) + menuProperties.VertMargin;    % Y position of the first item
    else
        thisTextYPos(i) = thisTextYPos(i-1) + thisTextBounds(4)-thisTextBounds(2) + ...
            menuProperties.EntryVertSpacing;    % Update y position
    end
    
    % ----------------------------------------------
    thisTextRect(i, :) = [thisTextXPos(i) thisTextYPos(i) thisTextXPos(i) thisTextYPos(i)] + ...
                    thisTextBounds;
end
%% Write the Texts
menuStatusChanged = 1;
menuRespondToKeypress = 1;
menuRefractoryClock = 0;

menuFirstKeyReleased = 0;
while 1
    
    if menuStatusChanged == 1
        % thisTextYPos = menuPos(2) + menuProperties.VertMargin;   % Starting Position
        % Change the blending mode so that the pixel with alpha~=1 would be
        % drawn properly
        refreshMenu(w, menuOffWin, menuRect, menuProperties, menuEntries, nEntries, menuCurrEntry, thisTextRect, thisTextXPos, thisTextYPos);   %(w, menuEntries, menuProperties, nEntries, menuCurrEntry, thisTextRect, menuOffWin, menuRect, FRAMERECTCONST)
        

        menuStatusChanged = 0;
        Screen('Flip',w, [], 1);
    end
    [keyDown sec keyCode] = KbCheck;
    
    if ~keyDown
        keyEverReleased = 1;
        menuFirstKeyReleased = 1;   % Every time a menu is created, you must release all the keys
                                    % for the menu to respond to keypresses
    end
    if (menuRespondToKeypress == 1||keyEverReleased == 1) && menuFirstKeyReleased == 1
        if keyCode(KbName('uparrow'))
            menuCurrEntry = wrapEntry(menuCurrEntry - 1, nEntries);
            while ~menuEntries(menuCurrEntry).Enabled
                menuCurrEntry = wrapEntry(menuCurrEntry - 1, nEntries);
            end
            menuStatusChanged = 1;
            menuRefractoryClock = GetSecs;
            menuRespondToKeypress = 0;  % Reset the clock
            keyEverReleased = 0;
        elseif keyCode(KbName('downarrow'))
            menuCurrEntry = wrapEntry(menuCurrEntry + 1, nEntries);

            while ~menuEntries(menuCurrEntry).Enabled
                menuCurrEntry = wrapEntry(menuCurrEntry + 1, nEntries);
            end
            menuStatusChanged = 1;
            menuRefractoryClock = GetSecs;
            menuRespondToKeypress = 0;
            keyEverReleased = 0;
        elseif keyCode(KbName('rightarrow'))
            if strcmp(menuEntries(menuCurrEntry).Type, 'enum')
                menuEntries(menuCurrEntry).CurrValue = GetGlobalValue(menuEntries(menuCurrEntry).BindedVariable);
                menuEntries(menuCurrEntry).CurrValue = min(menuEntries(menuCurrEntry).CurrValue+1, menuEntries(menuCurrEntry).Limits(2));
                
                eval(['global ' menuEntries(menuCurrEntry).BindedVariable '; ' menuEntries(menuCurrEntry).BindedVariable ' = menuEntries(menuCurrEntry).CurrValue;']);
                menuStatusChanged = 1;
                menuRefractoryClock = GetSecs;
                menuRespondToKeypress = 0;
                keyEverReleased = 0;
                
                % In some special cases, the modification made in the menu
                % must be rendered instantaneously (e.g. music volume).
                % Using the ReturnCode field to set the volume after the
                % value has been changed.
                if isfield(menuEntries(menuCurrEntry), 'ReturnCode')
                    eval(menuEntries(menuCurrEntry).ReturnCode);
                end
            
            end       
        elseif keyCode(KbName('leftarrow'))
            if strcmp(menuEntries(menuCurrEntry).Type, 'enum')
                menuEntries(menuCurrEntry).CurrValue = GetGlobalValue(menuEntries(menuCurrEntry).BindedVariable);
                menuEntries(menuCurrEntry).CurrValue = max(menuEntries(menuCurrEntry).CurrValue-1, menuEntries(menuCurrEntry).Limits(1));
                eval(['global ' menuEntries(menuCurrEntry).BindedVariable '; ' menuEntries(menuCurrEntry).BindedVariable ' = menuEntries(menuCurrEntry).CurrValue;']);

                menuStatusChanged = 1;
                menuRefractoryClock = GetSecs;
                menuRespondToKeypress = 0;
                keyEverReleased = 0;
                
                % In some special cases, the modification made in the menu
                % must be rendered instantaneously (e.g. music volume).
                % Using the ReturnCode field to set the volume after the
                % value has been changed.
                if isfield(menuEntries(menuCurrEntry), 'ReturnCode')
%                     global MusicSource
%                     alSourcef(MusicSource, AL.GAIN, MusicVolume./10);
                     eval(menuEntries(menuCurrEntry).ReturnCode);
                end
            
            end       
        elseif any(keyCode(KbName('return')))    % If the enter key is pressed
           %disp('ENTER pressed!');
            
            if strcmp(menuEntries(menuCurrEntry).Type, 'return')
                % Return to caller
                returnEntry = menuCurrEntry;
                returnCode = menuEntries(menuCurrEntry).ReturnCode;
                % Restore the old screen
                Screen('BlendFunction', w, GL_ONE, GL_ZERO);
                Screen('DrawTexture', w, menuOffWin, [], menuRect);
                Screen('BlendFunction', w, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

                return;
            elseif strcmp(menuEntries(menuCurrEntry).Type, 'subfunc')  
                % Execute several statements, then come back to this
                % function. menuEntries().ReturnCode should be a one liner
                % if is a string, or multi liner if it is a cell.
                Screen('BlendFunction', w, GL_ONE, GL_ZERO);
                Screen('DrawTexture', w, menuOffWin, [], menuRect);
                Screen('BlendFunction', w, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                if ischar(menuEntries(menuCurrEntry).ReturnCode)
                    eval(menuEntries(menuCurrEntry).ReturnCode);
                else iscell(menuEntries(menuCurrEntry).ReturnCode)
                    cmd = menuEntries(menuCurrEntry).ReturnCode;
                    for j = 1:length(menuEntries(menuCurrEntry).ReturnCode)
                        eval(cmd{j});
                    end
%                         rush(menuEntries(menuCurrEntry).ReturnCode, 0);
                end                 
                continue;
            elseif strcmp(menuEntries(menuCurrEntry).Type, 'enum')

            elseif strcmp(menuEntries(menuCurrEntry).Type, 'submenu')
                % If the parent menu must be cleared
                if menuEntries(menuCurrEntry).SubMenuProperties.ClearParent
                    Screen('BlendFunction', w, GL_ONE, GL_ZERO);
                    Screen('DrawTexture', w, menuOffWin, [], menuRect);
                    Screen('BlendFunction', w, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
                end
                [sub_returnEntry sub_returnCode] = showMenu(menuEntries(menuCurrEntry).SubMenuEntries, ...
                                                    menuEntries(menuCurrEntry).SubMenuProperties);
                if ischar(sub_returnCode)
                    eval(sub_returnCode);
                else iscell(sub_returnCode)
                    cmd = sub_returnCode;
                    for i = 1:length(cmd)
                         eval(cmd{i});
                    end
                    if strncmp(cmd{i}, 'return', 6);    % If it is a return   
%                         fprintf('Return Reached!\n');
%                         fprintf('ReturnEntry = %d, ReturnCode = %s\n', returnEntry, returnCode);
                        break;
                    end
%                         rush(menuEntries(menuCurrEntry).ReturnCode, 0);
                end       
                menuStatusChanged = 1;
                menuRefractoryClock = GetSecs;
                menuRespondToKeypress = 0;
                keyEverReleased = 0;
                menuFirstKeyReleased = 0;
            elseif strcmp(menuEntries(menuCurrEntry).Type, 'setkey')
                menuEntries(menuCurrEntry).CurrValue = []; 
                 refreshMenu(w, menuOffWin, menuRect, menuProperties, menuEntries, nEntries, menuCurrEntry, thisTextRect, thisTextXPos, thisTextYPos);
                 Screen('Flip',w,[],1);
                while KbCheck   % Wait until all keys (especially 'return') are released
                end
                while 1
                    [keyDown sec keyCode] = KbCheck;
                    if keyDown
                        menuEntries(menuCurrEntry).CurrValue = find(keyCode, 1,'first');    % Find the pressed key
                        break;
                    end
                end
                while KbCheck   % Again, wait for the key to release
                end
                eval(['global ' menuEntries(menuCurrEntry).BindedVariable '; ' menuEntries(menuCurrEntry).BindedVariable ' = menuEntries(menuCurrEntry).CurrValue']);
                
                menuStatusChanged = 1;
                menuRefractoryClock = GetSecs;
                menuRespondToKeypress = 0;  % Reset the clock
                keyEverReleased = 0;
            end
        elseif keyCode(KbName('escape')) && menuProperties.EnableEscape
            returnEntry = menuProperties.DefaultReturnEntry;
            returnCode = ';';
             Screen('BlendFunction', w, GL_ONE, GL_ZERO);
                    Screen('DrawTexture', w, menuOffWin, [], menuRect);
                    Screen('BlendFunction', w, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
            break;
        end
    else
        if GetSecs - menuRefractoryClock > menuProperties.RefractoryTime
            menuRespondToKeypress = 1;
        end
    end
end
end

%% Update the Menu
function refreshMenu(w, menuOffWin, menuRect, menuProperties, menuEntries, nEntries, menuCurrEntry, thisTextRect, thisTextXPos, thisTextYPos)
Screen('BlendFunction', w, GL_ONE, GL_ZERO);

Screen('DrawTexture', w, menuOffWin, [], menuRect);

Screen('BlendFunction', w, GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);


Screen('FillRect', w, menuProperties.MenuBkgColor, menuRect);
Screen('FrameRect', w, menuProperties.MenuFrameColor, menuRect);

for i = 1:nEntries
    Screen('TextFont', w, menuEntries(i).Font);
    Screen('TextSize', w, menuEntries(i).Size);
    if menuCurrEntry == i   % For the highlighted items
        Screen('FillRect', w, menuProperties.HighLightedBkgColor, thisTextRect(i, :));
        [newx newy] = Screen('DrawText', w, menuEntries(i).Caption, thisTextXPos(i), thisTextYPos(i), ...
            menuProperties.HighLightedTextColor);
        if strcmp(menuEntries(i).Type, 'enum')
            menuEntries(i).CurrValue = GetGlobalValue(menuEntries(i).BindedVariable);
            Screen('DrawText', w, num2str(menuEntries(i).CurrValue),  newx + menuEntries(i).Spacing, thisTextYPos(i), ...
                menuProperties.HighLightedTextColor);
        elseif strcmp(menuEntries(i).Type, 'setkey') && ~isempty(menuEntries(i).CurrValue);
            % If CurrValue == [], do not show any key names
            menuEntries(i).CurrValue = GetGlobalValue(menuEntries(i).BindedVariable);
            Screen('DrawText', w, KbName(menuEntries(i).CurrValue),  newx + menuEntries(i).Spacing, thisTextYPos(i), ...
                menuProperties.HighLightedTextColor);   % Use highlight color to show key name
        end
    else
        [newx newy] = Screen('DrawText', w, menuEntries(i).Caption, thisTextXPos(i), thisTextYPos(i), ...
            menuEntries(i).Color);
        if strcmp(menuEntries(i).Type, 'enum')  % For the Enumerate Type
            menuEntries(i).CurrValue = GetGlobalValue(menuEntries(i).BindedVariable);
            Screen('DrawText', w, num2str(menuEntries(i).CurrValue),  newx + menuEntries(i).Spacing, thisTextYPos(i), ...
                menuEntries(i).Color);
        elseif strcmp(menuEntries(i).Type, 'setkey') && ~isempty(menuEntries(i).CurrValue)
            menuEntries(i).CurrValue = GetGlobalValue(menuEntries(i).BindedVariable);
            Screen('DrawText', w, KbName(menuEntries(i).CurrValue),  newx + menuEntries(i).Spacing, thisTextYPos(i), ...
                menuProperties.HighLightedTextColor);
        end
    end
end
end

function xpos = horiCenterTextPos(textBounds, textureRect)
xpos = ((textureRect(3) - textureRect(1)) - ...
       (textBounds(3) - textBounds(1)))./2;
end

function newentry = wrapEntry(entry, entryLim)
if length(entryLim) == 1
    entryLim = [1 entryLim];
end
    
if entry<entryLim(1)
    newentry = entryLim(2);
elseif entry > entryLim(2)
    newentry = entryLim(1);
else
    newentry = entry;
end
end



    

    