function tetrisEnvironment
% TETRISENVIRONMENT    - Game Settings of Tetris that are not supposed to
% be changed by users

% Note:
% 1. All coordinate pairs, unless otherwise specified, shall be expressed in the 
%    form of [x y]
% 2. All coordinate pairs start from [1 1] instead of [0 0]

% Directories and Paths

%% Context:
global TETRIS_CONTEXT;     % Whether this installation of game is for gamers or 
                        % for psychophysists. 
global ResFactor;       % The resolution factor: all objects will be expanded by this much
global winRect;

%% Variable Demonstration:

global w;
global woff_whole;

global Main_Dir;
global Graph_Dir;
global Music_Dir;
global Setting_Dir;


global Music_Files;
global MusicFileNum;            % Number of in-game music files
% Monitor Settings
global MonitorNo;               % Which monitor to use
global ExpectedScreenSettings;  % The expected screeng setting, a structure 
                                % returned by Screen('Resolution');

global Flip_Interval;           % Interval between Physical Refreshes of the Monitor
global Frame_Interval;          % Interval between frames

% Play Field
global Playfield_1P;
global Playfield_Color_1P;

% One Block
global BLOCK_SIZE;

% Colors
global color_o;
global color_i;
global color_l;
global color_j;
global color_s;
global color_z;
global color_t;
global COLOR_WHEN_FIXED;         % When a block is fixed
global PF_BACKGROUND_COLOR;     % Bkgd color of the playfield

global TetradColor;

% Graphical Textures
global tex_BlockShade;      % The Shade of Blocks

% Scene Layouts
global PF_TOPXY_INT;     % The top left coordination of the playing field (inside)
global PF_TOPXY_EXT;     % The top left coordination of the playing field (outside)
global PF_WIDTH_INT;     % the width of the playing field (inside)
global PF_WIDTH_EXT;
global PF_HEIGHT_INT;    % the width of the playing field (outside)
global PF_HEIGHT_EXT;

global InitialX;

% These are [x y] pairs for prompting texts
global SCOREBOARDWHOLE_XY
global SCOREBOARDWHOLE_SIZE

% --- These constants are obsolete ---
global NEXTTETRADPROMPT_XY
global NEXTTETRADPROMPT_SIZE
global CURRENTLEVELPROMPT_XY
global CURRENTLEVELPROMPT_SIZE

global SCOREBOARD_XY
global SCOREBOARD_SIZE;
global LINESCLEARED_XY;
global LINESCLEARED_SIZE;
global LINESTOCLEAR_XY;
global LINESTOCLEAR_SIZE;
% --------------------------------------

% These are [x y] pairs for statistic texts
global TEXT_CLEARED_LINES_XY;
global TEXT_CLEARED_FONT 
global TEXT_CLEARED_SIZE 
global TEXT_CLEARED_COLOR

global TEXT_LINES_TO_CLEAR_FONT
global TEXT_LINES_TO_CLEAR_SIZE
global TEXT_LINES_TO_CLEAR_XY;
global TEXT_LINES_TO_CLEAR_COLOR
global TEXT_SCORE_XY;
global TEXT_SCORE_FONT;
global TEXT_SCORE_SIZE;
global TEXT_SCORE_COLOR;

global NEXT_TETRAD_XY
global CURRENT_LEVEL_XY
global CURRENT_LEVEL_FONT
global CURRENT_LEVEL_SIZE
global CURRENT_LEVEL_COLOR;


global ELAPSED_TIME_XY
global ELAPSED_TIME_FONT
global ELAPSED_TIME_SIZE
global ELAPSED_TIME_COLOR


% Block-related Configurations 
global MOVING_BLOCK         % The representative figure of a 'moving' block
global SETTLED_BLOCK        % The representative figure of a 'settled' block
global REFPOINT_ON_BLOCK    % The reference point on the block
                            % e.g. If I say: "The block is at [5 12]". It
                            % means that the REFPOINT is at [5 12], and the 
                            % location of the rest parts of the block will 
                            % be set accordingly. 
                            
% Tetromino clut index

global TetrisTypeIndex;
global TetrisClut;
global OldClut;

% Tetromino types
global Tetrad_O;
global Tetrad_I;
global Tetrad_J;
global Tetrad_L;
global Tetrad_S;
global Tetrad_Z;
global Tetrad_T;


% Speeds
global NormalFallSpeed;
global FreeFallSpeed;
global KeySampRate;
global HorizontalMoveCoolDownTime;       % The interval between two horizontal moves
                                        % if the Left or Right key is
                                        % pressed and held (in sec.).
global RotateCoolDownTime; 
global BlinkDuration;

% Text and their positions
global TextPause
global TextPsych
global PsychStatusStrings
global PsychLevelStrings

% Miscellaneous
global nMaxGameLevels
global nLinesForEachLevel   % The number of lines needed to cleared to pass each level
global GameRank             % The titles you'll be given based on your score
global GameRankScore

%% Context-Dependent Variables
global TimeLimit;
global EnableLevelUp;
global DataPath;
global TimerDisplay;

%% Variable Initialization

% GameContext = 'GAME';      % for experiments, or 'GAME' for pure gaming

Main_Dir = mfilename('fullpath');

thisFileName = mfilename;
Main_Dir = strrep(Main_Dir, thisFileName, '');

Graph_Dir = [Main_Dir  'Pics' filesep];         % Platform independent file separator
Music_Dir = [Main_Dir 'Music' filesep];
Setting_Dir = [Main_Dir 'Settings' filesep];

 Music_File = {'Opening.mp3', '1.mp3', '2.mp3', '3.mp3'};   
MusicFileNum = 3;
% Monitor Settings
% 

%% Load PsychSettings if the context is 'PSYCH'
if strcmpi(TETRIS_CONTEXT, 'psych')
    load([Setting_Dir  'PsychSettings.mat']);     % Load PsychSettings
    tetrisShowPsychSettings;        % Display the Psych Preferences
    tetrisSubjectInfo;              % Input the subject code and determine the name of the data file
    initPsychDataStruct;            % Initialize the data structure which will be saved later
elseif strcmpi(TETRIS_CONTEXT, 'gamer')
    load([Setting_Dir  'GamerSettings.mat']);
else
    error('tetrisEnvironment(): TETRIS_CONTEXT has to be either ''psych'' or ''gamer''');
end

[w winRect] = Screen('OpenWindow', MonitorNo, 0);
woff_whole = Screen('OpenOffScreenWindow', w, 0, winRect);

ResFactor = [winRect(3)./1024 winRect(4)./768];


try
    Flip_Interval = Screen('GetFlipInterval', w); 
catch
    Flip_Interval = 60;
end
Frame_Interval = Flip_Interval * 2;

% One Block
BLOCK_SIZE = [32 32].*ResFactor;


% Tetromino Clut Index
TetrisClutIndex = {'color_o', 'color_i', 'color_l', 'color_j',...
                   'color_s', 'color_z', 'color_t'};

% Colors
color_o = [0 0 255];
color_i = [255 0 0];
color_l = [255 0 255];
color_j = [255 255 0];
color_s = [0 255 255];
color_z = [255 165 0];
color_t = [0 255 0];

TetradColor = [
    color_o;
    color_i;
    color_l;
    color_j;
    color_s;
    color_z;
    color_t;
];
PF_BACKGROUND_COLOR = [128 128 128];

% TetrisClut = zeros(256,3);
% 
% for i = 1:7
%     TetrisClut(i,:) = eval(TetrisClutIndex{i});
% end

% Tetris Type Index
TetrisTypeIndex = ['o','i','l','j','s','z','t'];

% Scene Layouts
PF_TOPXY_INT = [150, 80].*ResFactor; %[380 84];     
PF_TOPXY_EXT = PF_TOPXY_INT - BLOCK_SIZE;

PF_WIDTH_INT = 10;     % the width of the playing field in 'blocks'(inside)
% PF_WIDTH_EXT = 12;
PF_HEIGHT_INT = 20;    % the height of the playing field in 'blocks' (inside)
% PF_HEIGHT_EXT = 22;
InitialX = 5;

SCOREBOARDWHOLE_XY = [512 0].*ResFactor;
SCOREBOARDWHOLE_SIZE = [1 1].*ResFactor;   % No scaling

% These constants are obsolete ----
NEXTTETRADPROMPT_XY = [600 100].*ResFactor;
NEXTTETRADPROMPT_SIZE = [0.7 0.7].*ResFactor;

CURRENTLEVELPROMPT_XY = [600 187].*ResFactor; 
CURRENTLEVELPROMPT_SIZE = [0.7 0.7].*ResFactor;

SCOREBOARD_XY = [600 330].*ResFactor;
SCOREBOARD_SIZE = [0.7 0.7].*ResFactor;    % The normalized size, 1 = original size
LINESCLEARED_XY = [600 465].*ResFactor;
LINESCLEARED_SIZE = [0.7 0.7].*ResFactor;  % x size, y size
LINESTOCLEAR_XY = [600 650].*ResFactor;
LINESTOCLEAR_SIZE = [0.7 0.7].*ResFactor;
% ----------------------------------

TEXT_CLEARED_LINES_XY = [720 485].*ResFactor;
TEXT_CLEARED_FONT = 'Helvetica';
TEXT_CLEARED_SIZE = round(30*ResFactor(1));
TEXT_CLEARED_COLOR = [255 255 255];
TEXT_LINES_TO_CLEAR_XY = [720 670].*ResFactor;
TEXT_LINES_TO_CLEAR_FONT = 'Helvetica';
TEXT_LINES_TO_CLEAR_SIZE = round(30*ResFactor(1));
TEXT_LINES_TO_CLEAR_COLOR = [255 255 255];

TEXT_SCORE_XY = [720 345].*ResFactor;
TEXT_SCORE_FONT = 'Arial Black';
TEXT_SCORE_SIZE = round(24*ResFactor(1));
TEXT_SCORE_COLOR = [128 128 255];

NEXT_TETRAD_XY = [690 20].*ResFactor;

CURRENT_LEVEL_XY = [750 195].*ResFactor;
CURRENT_LEVEL_FONT = 'Arial Black';
CURRENT_LEVEL_SIZE = round(round(36.*0.7)*ResFactor(1));
CURRENT_LEVEL_COLOR = [9 181 255];

ELAPSED_TIME_XY = [0 0];
ELAPSED_TIME_FONT = 'Helvetica';
ELAPSED_TIME_SIZE = round(16.*ResFactor(1));
ELAPSED_TIME_COLOR = [255 255 255];

% Block-related Configurations 
MOVING_BLOCK = 1;
SETTLED_BLOCK = 2;
REFPOINT_ON_BLOCK.X = 2;
REFPOINT_ON_BLOCK.Y = 1;

% Play Field
Playfield_1P = zeros(PF_HEIGHT_INT, PF_WIDTH_INT);
Playfield_Color_1P = zeros(PF_HEIGHT_INT, PF_WIDTH_INT);


% Speeds
% [NormalFallSpeed KeySampRate HorizontalMoveCoolDownTime RotateCoolDownTime] = getFallingSpeed(currGameLevel);
% NormalFallSpeed = 4;        % The ordinary falling speed (in block/sec)
FreeFallSpeed = 40;         % The free falling speed (in block/sec)
% KeySampRate = 40;                   % This many keys shall be processed in one sec.
% HorizontalMoveCoolDownTime = 0.12;       % The interval between two horizontal moves
%                                         % if the Left or Right key is
%                                         % pressed and held (in sec.).
% RotateCoolDownTime = 0.5; % The interval between two turnovers (in sec.).
BlinkDuration = 0.2;
% Playfield_1P(1:20,:) = SETTLED_BLOCK;
% Playfield_Color_1P(1:14,:) = 7;
% Playfield_Color_1P(15,:) = 1;
% Playfield_Color_1P(16,:) = 2;
% Playfield_Color_1P(17,:) = 3;
% Playfield_Color_1P(18,:) = 4;
% Playfield_Color_1P(19,:) = 5;
% Playfield_Color_1P(20,:) = 6;

% pf_side_wid = (PF_WIDTH_EXT - PF_WIDTH_INT)/2;   % The width of the sidebar
% pf_side_hgt = (PF_HEIGHT_EXT-PF_HEIGHT_INT)/2;   % The height of the horizontal bar

% Playfield_1P([1:pf_side_hgt,PF_HEIGHT_INT+pf_side_hgt+1:end], :) = 2;
% Playfield_1P(:, [1:pf_side_wid,PF_WIDTH_INT+pf_side_wid+1:end]) = 2;
    
% Tetromino types
Tetrad_O = zeros(4,4,4);
Tetrad_I = zeros(4,4,4);
Tetrad_J = zeros(4,4,4);
Tetrad_L = zeros(4,4,4);
Tetrad_S = zeros(4,4,4);
Tetrad_Z = zeros(4,4,4);
Tetrad_T = zeros(4,4,4);

Tetrad_O(:,:,1) = [1 1 0 0;
                   1 1 0 0;
                   0 0 0 0;
                   0 0 0 0];
Tetrad_O(:,:,2) = Tetrad_O(:,:,1);
Tetrad_O(:,:,3) = Tetrad_O(:,:,1);
Tetrad_O(:,:,4) = Tetrad_O(:,:,1);


Tetrad_I(:,:,1) = [0 1 0 0;
                   0 1 0 0;
                   0 1 0 0;
                   0 1 0 0];
Tetrad_I(:,:,2) = [0 0 0 0;
                   1 1 1 1;
                   0 0 0 0;
                   0 0 0 0];
Tetrad_I(:,:,3) = Tetrad_I(:,:,1);
Tetrad_I(:,:,4) = Tetrad_I(:,:,2);

Tetrad_J(:,:,1) = [0 1 0 0;
                   0 1 0 0;
                   1 1 0 0;
                   0 0 0 0];
Tetrad_J(:,:,2) = [0 0 0 0;
                   1 1 1 0;
                   0 0 1 0;
                   0 0 0 0];  
Tetrad_J(:,:,3) = [0 1 1 0;
                   0 1 0 0;
                   0 1 0 0;
                   0 0 0 0];
Tetrad_J(:,:,4) = [1 0 0 0;
                   1 1 1 0;
                   0 0 0 0;
                   0 0 0 0];
               
Tetrad_L(:,:,1) = [0 1 0 0;
                   0 1 0 0;
                   0 1 1 0;
                   0 0 0 0];
Tetrad_L(:,:,2) = [0 0 0 0;
                   1 1 1 0;
                   1 0 0 0;
                   0 0 0 0];  
Tetrad_L(:,:,3) = [1 1 0 0;
                   0 1 0 0;
                   0 1 0 0;
                   0 0 0 0];
Tetrad_L(:,:,4) = [0 0 1 0;
                   1 1 1 0;
                   0 0 0 0;
                   0 0 0 0];
              
Tetrad_S(:,:,1) = [0 1 1 0;
                   1 1 0 0;
                   0 0 0 0;
                   0 0 0 0];
Tetrad_S(:,:,2) = [0 1 0 0;
                   0 1 1 0;
                   0 0 1 0;
                   0 0 0 0];
Tetrad_S(:,:,3) = Tetrad_S(:,:,1);
Tetrad_S(:,:,4) = Tetrad_S(:,:,2);

Tetrad_Z(:,:,1) = [1 1 0 0;
                   0 1 1 0;
                   0 0 0 0;
                   0 0 0 0];
Tetrad_Z(:,:,2) = [0 0 1 0;
                   0 1 1 0;
                   0 1 0 0;
                   0 0 0 0];
Tetrad_Z(:,:,3) = Tetrad_Z(:,:,1);
Tetrad_Z(:,:,4) = Tetrad_Z(:,:,2); 
               
Tetrad_T(:,:,1) = [0 1 0 0;
                   1 1 1 0;
                   0 0 0 0;
                   0 0 0 0];
Tetrad_T(:,:,2) = [0 1 0 0;
                   0 1 1 0;
                   0 1 0 0;
                   0 0 0 0];
Tetrad_T(:,:,3) = [0 0 0 0;
                   1 1 1 0;
                   0 1 0 0;
                   0 0 0 0];  
Tetrad_T(:,:,4) = [0 1 0 0;
                   1 1 0 0;
                   0 1 0 0;
                   0 0 0 0];       


% Text and their positions
TextPause.String = 'PAUSED';
TextPause.Font = 'Helvetica';
TextPause.Size = round(24.*ResFactor(1));
TextPause.Position = [250 300].*ResFactor;
TextPause.Color = [255 255 255];
TextPause.BkgColor = [0 0 0];

PsychStatusStrings = {' ', 'DOUBLE ', 'TRIPLE ', 'SUPER '};
PsychLevelStrings = {' ', 'DUO!', 'TRIO!', 'PSYCHO!'};

TextPsych.String = [''];
TextPsych.Font = 'Helvetica';
TextPsych.Size = 18.*ResFactor(1);
TextPsych.Position = [0 0].*ResFactor;
TextPsych.Color = [0 255 0];
TextPsych.BkgColor = [0 0 0];
TextPsych.LifeTime = 0;
TextPsych.PosDiff = [0 -25].*ResFactor;    % This much of displacement will be made before it disappears

% Number of lines you have to clear to pass each levels
% nLinesForEachLevel = [1 1 1 1 1 1 1 1 1 1];
nLinesForEachLevel = [5 10 25 30 35 40 35 30 30 30];
% [5 10 25 30 35 40 35 30 30 30];
nMaxGameLevels = numel(nLinesForEachLevel);
totalLines = sum(nLinesForEachLevel);
%   lower bound              upper bound                title
GameRank = {...
'A CHEATER!';
'A NOOB';
 'AN AMATEUR';
'NOT THAT BAD';
 'SKILLED';
'A PRO';
 'A MASTER';
 'THE GRANDMASTER';
'GODLIKE!';
'HOLY SHIT!';
'... ...';
    };
GameRankScore = [0,totalLines, 3*totalLines, 6*totalLines,10*totalLines, 100*totalLines, ...
    1000*totalLines, 10000*totalLines,100000*totalLines,1000000*totalLines, 10000000*totalLines];
    
%% Menu Related Globals
global MusicVolume
global SoundVolume
global theLevelPlayerStartsFrom

% Keycodes
global LeftKey;
global RightKey;
global DownKey;
global TurnKey;
global EscKey;
global PauseKey;
global ResumeKey;

% load everything
eval(['load(''' Setting_Dir 'GlobalSetting.mat'');']); 
if ~EnableLevelUp
 nLinesForEachLevel = ones(10).*1000000;  
end