function tetrisMain()
% TETRISMAIN    - Main Game Program of Tetris
% Usage:
% ======
% userTimeLimit  = the 
% 
% 
% 
clear all;
%% Global Variables
global w;
global winRect;
global woff_whole;

global TETRIS_CONTEXT;

global color_o;
global color_i;
global color_l;
global color_j;
global color_s;
global color_z;
global color_t;

global MonitorNo;           % The monitor u wish to play the game on
global Flip_Interval;
global Frame_Interval;

global Playfield_1P;
global Playfield_Color_1P;

global REFPOINT_ON_BLOCK;

global PF_TOPXY_INT;        % The top left coordination of the playing field (inside)
global PF_TOPXY_EXT;        % The top left coordination of the playing field (outside)

global PF_WIDTH_INT;        % The width of the playing field (inside)
global PF_HEIGHT_INT;       % The height of the playing field (inside)

global BLOCK_SIZE;

global Tetrad_O;        % The shape matrices of the tetrads
global Tetrad_I;
global Tetrad_J;
global Tetrad_L;
global Tetrad_S;
global Tetrad_Z;
global Tetrad_T;

global InitialX;

global LeftKey;
global RightKey;
global DownKey;
global TurnKey;
global EscKey;
global PauseKey;
global ResumeKey;

global NormalFallSpeed;
% The speed a block would fall without any key pressed. This is the
% 'normal' speed.

global FreeFallSpeed;
% The speed a block would fall with 'downKey' pressed.

global KeySampRate;      % Keypressing would be processed this many times per sec.
global HorizontalMoveCoolDownTime;       % The interval between two horizontal moves
% if the Left or Right key is
% pressed and held (in sec.).
global RotateCoolDownTime;

global TextPause;
global TextPsych;
global PsychStatusStrings;
global PsychLevelStrings

global nLinesForEachLevel

global nMaxGameLevels 
%
global menuProperties
global menuEntries
global inGameMenuProperties
global inGameMenuEntries
global theLevelPlayerStartsFrom;

global MusicBuffer

global OpeningMusicBuffer

global CreditsMusicBuffer
global CreditsScrollingTime     % Time it takes to finish the entire scroll

global GameOverMusicBuffer

global MusicSource

global MusicFileNum

global MusicVolume
global SoundVolume

% Time control
global TimeLimit
global DataPath
global EnableLevelUp
global timerLimitStart      % The Global Timer


% Whether the Timer is visible
global TimerDisplay

 % If the player loses before TimeLimit is reached, whether to reset timer
 % when a new round starts
global ResetTimerIfPlayerLoses

 % If the player loses before TimeLimit is reached, whether to reset score
 % when a new round starts
global ResetScoreIfPlayerLoses

global elapsedTime

global AL;
%% For psychophysicists
global PSYCHDATA    % The Backup Timer, Used to keep the real start time 
global SUBJECT_CODE;

% Get the Game's Context

% gameContext = 'gamer';
% if ~isempty(gameContext)
%     if ischar(gameContext)
%         if strcmpi(gameContext, 'psych') || strcmpi(gameContext, 'gamer')
%             TETRIS_CONTEXT = gameContext;
%         else
%             error('tetrisMain(): Parameter ''gameContext'' should be either ''psych'' or ''gamer''');
%         end
%     else
%         error('tetrisMain(): Parameter ''gameContext'' should be a string');
%     end
% else
TETRIS_CONTEXT = 'psych';   % By default, the CONTEXT is 'gamer'
% end

ResetTimerIfPlayerLoses = false;
ResetScoreIfPlayerLoses = false;

% if strcmpi(TETRIS_CONTEXT, 'psych')
%     TimerDisplay = true;
% else
%     TimerDisplay = false;
% end

% 
% Main_Dir = mfilename('fullpath');
% 
% thisFileName = mfilename;
% Main_Dir = strrep(Main_Dir, thisFileName, '');  % Get the path


%% --Start of the Main Program --

oldScreenSettings = tetrisCheckScreenSettings;
% Used to save the screen when needed

% Initialize variables and load resources before the real game starts
tetrisEnvironment;                          % Setup Tetris Environment
tetrisLoader;                               % Load resources
tetrisCopyright;                            % Display Copyright info;

mainCycleFirstEntry = 1;
while 1           % Level 1 Cycle: Overall
    
    if mainCycleFirstEntry == 1
        mainCycleFirstEntry = 0;
    else     % If this is not the first time the menu is shown (e.g. quit from the game)
        % replay the title music
        % The steps of changing music
        % 1: Stop the source
        % 2: Unqueue the source
        % 3: Queue the source with a new buffer
        % 4: Play the source
% %         alSourceStop(MusicSource);
% %         alSourceUnqueueBuffers(MusicSource, 1);
% %         alSourceQueueBuffers(MusicSource, 1, OpeningMusicBuffer);
% %         alSourcei(MusicSource, AL.LOOPING, AL.TRUE);
% %         alSourcePlay(MusicSource);
    end
tetrisTitleScreen;
menuInit;       % Re-init the menu
[returnEntry returnCode] = showMenu(menuEntries, menuProperties);   % Display the Title Screen and
if strncmp(returnCode, 'return', 6) % If an non-exist item is selected
    ListenChar(1);
    Screen('CloseAll');
    tetrisUnloader(oldScreenSettings);
    clear all;
    return
end

Screen('Flip', w, [], 0);
initPsychDataStruct;    % Initialize the PSYCHDATA structure

% saveGameSettings;       % Save User Settings so that the user can restore later

% Main Menu
%% Some useful variables, including the scoreboard, history of cleared
%% lines, etc.
% % alSourceStop(MusicSource);
% % alSourceUnqueueBuffers(MusicSource, 1);
% % alSourceQueueBuffers(MusicSource, 1, MusicBuffer);
% % alSourcePlay(MusicSource);

currScore = uint32(0);          % Make it 64 bits in case the user gets very high scores
lastScore = currScore;
lastScoreIncrement = 1;

clearedLineHistory = zeros(1,4);    % keeps four previous line-clearing action
clearedLineHistory_Ptr = 0;
     % The user chooses to start from ...
currGameLevel = theLevelPlayerStartsFrom;
[NormalFallSpeed KeySampRate HorizontalMoveCoolDownTime RotateCoolDownTime] = getFallingSpeed(currGameLevel);
nTotalClearedLines = 0;                 % How many levels cleared since the start of the game
nMoreLinesBeforeNextLevel = nLinesForEachLevel(theLevelPlayerStartsFrom);
% Just give it a number if user chooses to start from level 1

nLinesForEachLevelCurr = nLinesForEachLevel(theLevelPlayerStartsFrom:end);

currPsychoStatus = 1;               % PsychoStatus, which affects the point you get.

currComboTimes = 0;         % How many combos you make after "SUPER" combos
currPsychoLines = 1;
currPsychString = [];

lastComboTimes = 0;
lastPsychoLines = 1;

Playfield_1P = Playfield_1P .* 0;                  % Clear the playfield
Playfield_Color_1P = Playfield_Color_1P .* 0;
%% End of Initialiation

curFallSpeed = NormalFallSpeed;             % Get the initial falling speed
drawPFOutline;

% Get the First Tetrad
[nextTetradMatrix nextTetradIndex nextTetradOri nextTopMostLine nextColor] = genNewTetrad;

%% Main Loop
timerLimitStart = GetSecs;           % Start the 'Limit' timer

PSYCHDATA.DATE = datestr(date);     % Gets the date
PSYCHDATA.STARTTIME = datestr(clock, 'HH:MM:SS');
PSYCHDATA.SUBJECTCODE = SUBJECT_CODE;

lastDownKeyPressed = 0;         % whether the 'DownKey' was pressed in the last cycle
while 1            % Level 2 Cycle: Main
    % Get the current tetrad
    [curTetradMatrix curTetradIndex curTetradOri topMostLine curColor] = ...
        deal(nextTetradMatrix, nextTetradIndex, nextTetradOri, nextTopMostLine, nextColor);

    % Make the next Tetrad
    [nextTetradMatrix nextTetradIndex nextTetradOri nextTopMostLine nextColor] = genNewTetrad;

    [curTetradX, curTetradY] = deal(InitialX, 2-topMostLine);  % The inital position

    drawPFOutline;
    redrawPlayfieldPF;
    drawTetrominoPF(curTetradMatrix, curTetradX, curTetradY, curColor);
% % %     if TextPsych.LifeTime>0;    % Which means this Text Structure is revived
% % %         Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]); % Clear the Text by covering it.
% % %         [TextPsych] = showText(TextPsych);
% % %         %          TextPsych.LifeTime = TextPsych.LifeTime - Flip_Interval; % Predicted LifeTime
% % %         %          TextPsych.Position = TextPsych.Position + TextPsych.PosDiff.*Flip_Interval;
% % %                  TextPsych.Color = TextPsych.Color - TextPsych.Color.*Flip_Interval;
% % %     elseif ~isempty(nonzeros(TextPsych.Color - TextPsych.BkgColor))
% % %         TextPsych.Color = TextPsych.BkgColor;
% % %         Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]);
% % %         %                 [TextPsych lastPsychTextPos] =
% showText(TextPsych);
% % %     end

    updateScoreBoard(nextTetradMatrix, nextColor, currScore,nTotalClearedLines,nMoreLinesBeforeNextLevel,currGameLevel);
    tetrisScoreBoard;
    
        % ELAPSED TIME
    elapsedTime = displayElapsedTime(timerLimitStart, GetSecs);
    lastVBL = Screen('Flip', w, [], 1);    % Get the time of the last flip
    % If timeLimit is reached, back to main menu
    if elapsedTime > TimeLimit && TimeLimit~=0; tetrisGameEnding(2, currScore); esc_flag = 0; break; end   
    
%     if checkTetradOverlapping(curTetradMatrix, curTetradX, curTetradY)
%         % TO-DO: Add Fail Check
% 
%         break;  % If the tetrad overlaps with existing blocks, quit
%     end
    fallStartTime = lastVBL;        % The time the block starts to fall
    keyRespondStartTime = lastVBL;  % The time the system starts to respond to keypresses
    lastLevelsFallen = 0;
    lastKeysResponded = 0;


    lastLeftKeyPressed = 0;         % whether the 'LeftKey' was pressed in the last cycle
    lastRightKeyPressed = 0;        % whether the 'RightKey' was pressed in the last cycle
    lastTurnKeyPressed = 0;         % whether the 'TurnKey' was pressed in the last cycle
    downKeyEverReleased = 0;        % whether the 'DownKey' has been released during the falling of a tetrad

    settled_flag = 0;               % if the tetrad has fallen to the bottom and become 'settled'
    esc_flag = 0;                   % whether the ESC key is pressed
    restart_flag = 0;
    
    timerHorizCoolDown_Start = 0;
    timerHorizCoolDown_End = 0;
    timerTurnoverCoolDown_Start = 0;
    timerTurnoverCoolDown_End = 0;

    timerFreeFall_Start = 0;
    timerFreeFall_End = 0;
    while 1     % Level 3 Cycle: One Tetrad
        %          drawPFOutline;
        %          redrawPlayfieldPF;
        nextVBL = lastVBL + Frame_Interval;     % The time the next frame is to be shown
        clearTetrominoPF(curTetradMatrix, curTetradX, curTetradY);
        [keyDown secs keyCode] = KbCheck;
        % ---------------------------------------------------------------------
        % OK, here's the logic: The program would respond to'DownKey' event only
        % ONCE at the moment the 'DownKey' is pressed.
        % If you hold the 'DownKey', the program will not respond at all until
        % the 'DownKey' is released.
        % if the downKey was not pressed in the last cycle.
        if lastDownKeyPressed == 0 || ~keyCode(DownKey)
            downKeyEverReleased = 1;    % Then downKey 'was' indeed released.
        end
        % if the downKey has never been released
        if downKeyEverReleased == 0
            keyCode(DownKey) = 0;       % Do not respond to DownKey
        end
        % -----------------------------------------------------------------

        %%
        if ~keyCode(DownKey)
%             fprintf('DownKey Released!\n');
            % If downkey was for the first time released
            % after a free fall ends.
            if lastDownKeyPressed && downKeyEverReleased
                % Only when the free Fall really starts will the timers be
                % restarted
                if freeFallCurLevel > 0
                    % Restart all timers
                    lastDownKeyPressed = 0;
                    fallStartTime = GetSecs;                % Get the time when the DownKey was released
                    lastLevelsFallen = 0;                   % How many levels have been fallen since fallStartTime
                    keyRespondStartTime = fallStartTime;
                    lastKeysResponded = 0;               % How many keys have been responded since fallStartTime
                end
            end
            % How many levels to fall and how many keys to respond before
            % the next flip;
            levelsToFall = floor((nextVBL - fallStartTime).* curFallSpeed - lastLevelsFallen);
            keysToRespond =floor((nextVBL - keyRespondStartTime).* KeySampRate - lastKeysResponded);

            for i = 1:keysToRespond     % Level 4 Cycle: KeyPresses
                [keyDown secs keyCode] = KbCheck;
                if ~keyDown                                 % If no keypress is detected
                    % Clear all keypressed flags
                    lastDownKeyPressed = 0;
                    lastLeftKeyPressed = 0;
                    lastRightKeyPressed = 0;
                    lastTurnKeyPressed = 0;
                    %                     downKeyEverReleased = 1;
                    continue;                               % process another keypress
                end
                if ~(keyCode(LeftKey)&&keyCode(RightKey))   % If both keys are pressed then do nothing
                    if keyCode(LeftKey)                     % If left key is pressed
                        if lastLeftKeyPressed == 0          % If left key was released in the last cycle
                            if isBlockMovable(curTetradX, curTetradY, curTetradMatrix, 'left');
                                curTetradX = curTetradX - 1;
                                timerHorizCoolDown_Start = GetSecs;  % Get the start of horiz movement
                            end
                        else
                            timerHorizCoolDown_End = GetSecs;
                            if timerHorizCoolDown_End - timerHorizCoolDown_Start >= HorizontalMoveCoolDownTime
                                if isBlockMovable(curTetradX, curTetradY, curTetradMatrix, 'left');
                                    curTetradX = curTetradX - 1;
                                    timerHorizCoolDown_Start = GetSecs;      %  Get the start of horiz movement
                                end
                            end

                        end
                        lastLeftKeyPressed = 1;
                        lastRightKeyPressed = 0;

                    elseif keyCode(RightKey)
                        if lastRightKeyPressed == 0          % If left key was released in the last cycle
                            if isBlockMovable(curTetradX, curTetradY, curTetradMatrix, 'right');
                                curTetradX = curTetradX + 1;
                                timerHorizCoolDown_Start = GetSecs;  % Get the start of horiz movement
                            end
                        else
                            timerHorizCoolDown_End = GetSecs;
                            if timerHorizCoolDown_End - timerHorizCoolDown_Start >= HorizontalMoveCoolDownTime
                                if isBlockMovable(curTetradX, curTetradY, curTetradMatrix, 'right');
                                    curTetradX = curTetradX + 1;
                                    timerHorizCoolDown_Start = GetSecs;      %  Get the start of horiz movement
                                end
                            end

                        end
                        lastLeftKeyPressed = 0;
                        lastRightKeyPressed = 1;
                    end
                end

                if keyCode(TurnKey)
                    if lastTurnKeyPressed == 0          % If TurnOver key was released in the last cycle
                        [curTetradMatrix curTetradOri changeFlag] = rotateTetromino...
                            (curTetradMatrix, curTetradX, curTetradY, curTetradOri, curTetradIndex);
                        timerTurnoverCoolDown_Start = GetSecs;  % Get the start of turnover
                        %                             fprintf('lastTurnKeyPressed=0;');
                    else
                        timerTurnoverCoolDown_End = GetSecs;
                        if timerTurnoverCoolDown_End - timerTurnoverCoolDown_Start >= RotateCoolDownTime
                            [curTetradMatrix curTetradOri changeFlag] = rotateTetromino...
                                (curTetradMatrix, curTetradX, curTetradY, curTetradOri, curTetradIndex);
                            timerTurnoverCoolDown_Start = GetSecs;  % Get the start of turnover
                            %                             fprintf('lastTurnKeyPressed=1;');
                        end
                        %                         fprintf('lastTurnKeyPressed=2;');
                    end
                    lastTurnKeyPressed = 1;
                else
                    lastTurnKeyPressed = 0;     % If there are keys pressed, but not TurnKey
                end
% 
%                 if keyCode(EscKey)
%                     esc_flag = 1;
%                     break;
%                 end
                if keyCode(PauseKey)
                    pause_flag = 1;
                else
                    esc_flag = 0;
                    restart_flag = 0;
                    pause_flag = 0;
                end
                
                if keyCode(EscKey)
                    ingamemenu_flag = 1;
                else
                    esc_flag = 0;
                    restart_flag = 0;
                    ingamemenu_flag = 0;
                end
                if pause_flag || ingamemenu_flag % If pause key or menu key (esc) is pressed
                    timerPause_Start = GetSecs;
                    % Must do this because PauseKey may = ResumeKey
                    while keyCode(PauseKey)||keyCode(EscKey)     % Wait until both Keys are released
                        [keyDown secs keyCode] = KbCheck;
                    end

                    % Display the 'Paused' string
                    
                    drawTetrominoPF(curTetradMatrix, curTetradX, curTetradY, curColor);
                    
                    if pause_flag
                        showText(TextPause);    %'PAUSED', 'Helvetica', 24, 'center', [255 255 255]);
                        Screen('Flip', w, [], 1);   % Show the text(without cleaning the last frame)
                        while 1
                            [keyDown secs keyCode] = KbCheck;
                            if keyCode(ResumeKey)
                                break;
                            end
                        end

                        % Wait until ResumeKey is released
                        while keyCode(ResumeKey)
                            [keyDown secs keyCode] = KbCheck;
                        end
                    elseif ingamemenu_flag
                        Screen('Flip', w, [], 1);   % Show the text(without cleaning the last frame)

                        [returnEntry returnCode] = showMenu(inGameMenuEntries, inGameMenuProperties);
%                         returnCode;
                        eval(returnCode);
                        [keyDown secs keyCode] = KbCheck;  
                        while keyCode(EscKey)
                            [keyDown secs keyCode] = KbCheck;                            
                        end

                    end
                    % Time to reset the pause_flag and ingamemenu_flag
                    pause_flag = 0;
                    ingamemenu_flag = 0;
                    if esc_flag||restart_flag
                        break;      % Now in Level 4 Cycle
                    end
                    % The 'PAUSED' text is shown before a tetrad is drawn,
                    % so you will not see the currently active tetrad
                    % during pausing.
                    %                 tetrisScoreBoard;
                    %                 updateScoreBoard(nextTetradMatrix, nextColor, currScore,nTotalClearedLines,nMoreLinesBeforeNextLevel,currGameLevel)
                    
                    % Keep waiting until ResumeKey is pressed

                    drawPFOutline;
                    redrawPlayfieldPF;
% % %                     if TextPsych.LifeTime>0;    % Which means this Text Structure is revived
% % %                         Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]); % Clear the Text by covering it.
% % %                         [TextPsych ] = showText(TextPsych);
% % %                         %          TextPsych.LifeTime = TextPsych.LifeTime - Flip_Interval; %
% % %                         %          Predicted LifeTime
% % %                         %          TextPsych.Position = TextPsych.Position + TextPsych.PosDiff.*Flip_Interval;
% % %                         %          TextPsych.Color = TextPsych.Color - TextPsych.Color.*Flip_Interval;
% % %                     elseif ~isempty(nonzeros(TextPsych.Color - TextPsych.BkgColor))
% % %                         TextPsych.Color = TextPsych.BkgColor;
% % %                         Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]);
% % %                     end
                    drawTetrominoPF(curTetradMatrix, curTetradX, curTetradY, curColor);
                    
                    
                    %        tetrisScoreBoard;
                    %        updateScoreBoard(nextTetradMatrix, nextColor, currScore,nTotalClearedLines,nMoreLinesBeforeNextLevel,currGameLevel)
                    nextVBL = Screen('Flip',w, [], 1);      %
                    if elapsedTime > TimeLimit && TimeLimit~=0; tetrisGameEnding(2, currScore); restart_flag = 0; break; end   
                    % This will make the 'PAUSED' text stay on the screen
                    % for at least one 1 extra frame after 'ResumeKey' is
                    % pressed. This is not a desirable feature but it is
                    % accepteable. I guess few people would really notice
                    % this.
                    timerPause_End = GetSecs;
                    pauseTime = timerPause_End - timerPause_Start;
                    
                    % subtract the pauseTime 
                    timerLimitStart = timerLimitStart + pauseTime;
                    fallStartTime = fallStartTime + pauseTime;
                    keyRespondStartTime = keyRespondStartTime + pauseTime;

                    if timerHorizCoolDown_Start == 0
                        timerHorizCoolDown_Start = timerHorizCoolDown_Start + pauseTime;
                    end
                    if timerTurnoverCoolDown_Start == 0
                        timerTurnoverCoolDown_Start = timerTurnoverCoolDown_Start + pauseTime;
                    end
                    if timerFreeFall_Start == 0
                        timerFreeFall_Start = timerFreeFall_Start + pauseTime;
                    end
                end

            end
            if esc_flag || restart_flag     % now in Level 3 Cycle
                break;
            end

            lastKeysResponded = lastKeysResponded + keysToRespond;

            for i = 1:levelsToFall      % Process fallings of multiple levels
                % If a tetrad does not reach the bottom
                if isBlockMovable(curTetradX, curTetradY, curTetradMatrix,'bottom');
                    curTetradY = curTetradY + 1;
                    lastLevelsFallen = lastLevelsFallen + 1;
                else
                    settled_flag = 1;
                    %                     fprintf('Block Ended!\n');
                    break;
                end
            end

            drawTetrominoPF(curTetradMatrix, curTetradX, curTetradY, curColor);

% % %             if TextPsych.LifeTime>0;    % Which means this Text Structure is revived
% % %                 Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]); % Clear the Text by covering it.
% % %                 [TextPsych ] = showText(TextPsych);
% % %                 %          TextPsych.LifeTime = TextPsych.LifeTime - Flip_Interval; % Predicted LifeTime
% % %                 %          TextPsych.Position = TextPsych.Position + TextPsych.PosDiff.*Flip_Interval;
% % %                 %          TextPsych.Color = TextPsych.Color - TextPsych.Color.*Flip_Interval;
% % %             elseif ~isempty(nonzeros(TextPsych.Color - TextPsych.BkgColor))
% % %                 TextPsych.Color = TextPsych.BkgColor;
% % %                 Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]);
% % % 
% % %             end

            %        updateScoreBoard(nextTetradMatrix, nextColor, currScore,nTotalClearedLines,nMoreLinesBeforeNextLevel,currGameLevel)
            %        tetrisScoreBoard;

            if settled_flag==1
                % ELAPSED TIME
                elapsedTime = displayElapsedTime(timerLimitStart, GetSecs);
                if elapsedTime > TimeLimit && TimeLimit~=0
                    lastVBL = Screen('Flip',w, nextVBL-0.5*Flip_Interval,1);
                    tetrisGameEnding(2, currScore);
                    restart_flag = 1;
                    break;
                end 
                
                lastVBL = Screen('Flip',w, nextVBL-0.5*Flip_Interval,0);
                break;
            else
                % ELAPSED TIME
                elapsedTime = displayElapsedTime(timerLimitStart, GetSecs);
                lastVBL = Screen('Flip',w, nextVBL-0.5*Flip_Interval,1);
                if elapsedTime > TimeLimit && TimeLimit~=0; tetrisGameEnding(2, currScore); restart_flag = 1; break;end   
            end
            %
            %                 if checkTetradOverlapping(curTetradMatrix, curTetradX, curTetradY) ||...
            %                         checkTetradOutBound(curTetradMatrix, curTetradX, curTetradY)


            %            if ~lastDownKeyPressed
            %                lastDownKeyPressed = 1;
            %                FreeFallStartTime = GetSecs;     % Get the Start Time of the free fall
            %                lastFreeFallLevels =
        else
            %              fprintf('DownKey Pressed!\n');
            if lastDownKeyPressed == 0     % if Down key was not pressed in the last cycle
                lastDownKeyPressed = 1;     % Start the Free Fall SEQUENCE!
                timerFreeFall_Start = GetSecs;  % Start the Free Fall Timer!
                freeFallCurLevel = 0;       % How many levels have the tetrad fallen since the
                freeFallLastLevel = 0;      % Timer starts
                %                 fprintf('FreeFallTimer Restarted!\n');
            end
            freeFallCurLevel = floor((nextVBL - timerFreeFall_Start).*FreeFallSpeed);
            levelsToFall = freeFallCurLevel - freeFallLastLevel;
            % How many levels to fall so that the tetrad will reach its
            % current position

            %             fprintf('levelsToFall: %f', levelsToFall);
            for i = 1:levelsToFall
                if isBlockMovable(curTetradX, curTetradY, curTetradMatrix,'bottom')
                    curTetradY = curTetradY + 1;
                    freeFallLastLevel = freeFallLastLevel + 1;
                else
                    settled_flag = 1;
                    %                     fprintf('Block ended!');
                    break;
                end
            end
            %             fprintf('%d Levels fallen\n', i);

            drawTetrominoPF(curTetradMatrix, curTetradX, curTetradY, curColor);
% % %             if TextPsych.LifeTime>0;    % Which means this Text Structure is revived
% % %                 Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]); % Clear the Text by covering it.
% % %                 [TextPsych ] = showText(TextPsych);
% % %                 %          TextPsych.LifeTime = TextPsych.LifeTime - Flip_Interval; % Predicted LifeTime
% % %                 %          TextPsych.Position = TextPsych.Position + TextPsych.PosDiff.*Flip_Interval;
% % %                 %          TextPsych.Color = TextPsych.Color - TextPsych.Color.*Flip_Interval;
% % %             elseif ~isempty(nonzeros(TextPsych.Color - TextPsych.BkgColor))
% % %                 TextPsych.Color = TextPsych.BkgColor;
% % %                 Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]);
% % %             end
            %        updateScoreBoard(nextTetradMatrix, nextColor, currScore,nTotalClearedLines,nMoreLinesBeforeNextLevel,currGameLevel)
            %        tetrisScoreBoard;

            if settled_flag==1  % If this tetrad is settled, clear the framebuffer
                % ELAPSED TIME
                elapsedTime = displayElapsedTime(timerLimitStart, GetSecs);
                if elapsedTime > TimeLimit && TimeLimit~=0
                    lastVBL = Screen('Flip',w, nextVBL-0.5*Flip_Interval,1); 
                    tetrisGameEnding(2, currScore); 
                    restart_flag = 1; 
                    break;
                end 
                
                lastVBL = Screen('Flip',w, nextVBL-0.5*Flip_Interval,0);
                
                break;
            else
                % ELAPSED TIME
                elapsedTime = displayElapsedTime(timerLimitStart, GetSecs);
                lastVBL = Screen('Flip',w, nextVBL-0.5*Flip_Interval,1);
                if elapsedTime > TimeLimit && TimeLimit~=0; tetrisGameEnding(2, currScore); restart_flag = 1; break;end   
            end
        end

    end

    if esc_flag || restart_flag         % Now in Level 2 Cycle
        restart_flag = 0;
        break;
    end
    %     Screen('Flip',w,[], 0);
    %% What happens after a tetrad is settled
    drawTetrominoPF(curTetradMatrix, curTetradX, curTetradY, curColor);
    piledup_flag = addTetradToPlayfield(curTetradMatrix, curTetradX, curTetradY, curTetradIndex);
    % If blocks are piled to the highest place, then break
    if piledup_flag
        WaitSecs(0.5);
        % Refresh the game scene
        drawPFOutline;
        redrawPlayfieldPF();
        updateScoreBoard(nextTetradMatrix, nextColor, currScore,nTotalClearedLines,nMoreLinesBeforeNextLevel,currGameLevel);
        tetrisScoreBoard
        
        if TimeLimit==0     % Normal situation, back to main menu
            tetrisGameEnding(0);
            esc_flag = 0;
            break;
        else                % Playing with time limit
            timerPause_Start = GetSecs;
            tetrisGameEnding(0);
            Screen('Flip',w);   % Clear the screen
            timerPause_End = GetSecs;
            pauseTime = timerPause_End - timerPause_Start;
            % Prepare to restart the game
            if elapsedTime < TimeLimit
                
                if ResetScoreIfPlayerLoses
                    currScore = uint32(0);          % Make it 64 bits in case the user gets very high scores
                    lastScore = currScore;
                end
              
%                 lastScoreIncrement = 1;
% 
%                 clearedLineHistory = zeros(1,4);    % keeps four previous line-clearing action
%                 clearedLineHistory_Ptr = 0;
%                 % The user chooses to start from ...
%                 currGameLevel = theLevelPlayerStartsFrom;
                [NormalFallSpeed KeySampRate HorizontalMoveCoolDownTime RotateCoolDownTime] = getFallingSpeed(currGameLevel);
%                 nTotalClearedLines = 0;                 % How many levels cleared since the start of the game
%                 nMoreLinesBeforeNextLevel = nLinesForEachLevel(theLevelPlayerStartsFrom);
                % Just give it a number if user chooses to start from level 1

%                 nLinesForEachLevelCurr = nLinesForEachLevel(theLevelPlayerStartsFrom:end);

                currPsychoStatus = 1;               % PsychoStatus, which affects the point you get.

                currComboTimes = 0;         % How many combos you make after "SUPER" combos
                currPsychoLines = 1;
                currPsychString = [];

                lastComboTimes = 0;
                lastPsychoLines = 1;

                Playfield_1P = Playfield_1P .* 0;                  % Clear the playfield
                Playfield_Color_1P = Playfield_Color_1P .* 0;
                %% End of Initialiation

                curFallSpeed = NormalFallSpeed;             % Get the initial falling speed

                % Get the First Tetrad
                if ResetTimerIfPlayerLoses
                    timerLimitStart = GetSecs;
                else
                    timerLimitStart = timerLimitStart + pauseTime;
                end
                
            end
            continue;
        end
    end

    % This part is buggy
    %% Find the filled lines, add scores etc.
    [filledLines, nLines] = findFilledLines(curTetradY-REFPOINT_ON_BLOCK.Y+1, min(curTetradY-REFPOINT_ON_BLOCK.Y+4, PF_HEIGHT_INT));

    WaitSecs(0.2);
%     TextPsych.LifeTime = 0; % Kill the string
    if ~isempty(filledLines)
        % Draw these things so that you don't have to redraw them in
        % 'deleteLineAnimation'
        updateScoreBoard(nextTetradMatrix, nextColor, currScore,nTotalClearedLines,nMoreLinesBeforeNextLevel,currGameLevel);
        tetrisScoreBoard
        deleteLineAnimation(filledLines);

        if clearedLineHistory_Ptr < 4    % If the Pointer becomes 4, stop increment.
            clearedLineHistory_Ptr = clearedLineHistory_Ptr + 1;
        else
            clearedLineHistory(1:3) = clearedLineHistory(2:4);
        end
        clearedLineHistory(clearedLineHistory_Ptr) = nLines;
        currPsychStatus = 0;     % initialize psychstatus
        % Get the current 'psychstatus'
        % If the only one line is cleared, display nothing on the screen
        % In this case, no matter how many repetitions you make, the psychstatus would not
        % change.
        if nLines == 1
            currPsychStatus = 1;
        else
            for i = clearedLineHistory_Ptr: -1: 1
                if clearedLineHistory(i) == nLines
                    currPsychStatus = currPsychStatus + 1;
                else
                    break;
                end
            end
        end
        % currPsychString is a text description of your current psych status
        currPsychLines = nLines;
% % %         TextPsych.String = {PsychStatusStrings{currPsychStatus}, ...
% % %             PsychLevelStrings{nLines}, ''};
% % % 
% % %         TextPsych.Position = [0, curTetradY.*BLOCK_SIZE(2)+PF_TOPXY_INT(2)]; % Rightmost of the playfield
% % %         TextPsych.Color = [0 255 0];
% % %         TextPsych.LifeTime = 1;  % Stay for two seconds

        lastPsychStatus = currPsychStatus;
        lastPsychLines = lastPsychStatus;

        if currPsychStatus == 4  % Combo counter started after PsychStatus reaches 4
            currComboTimes = currComboTimes + 1;
%             if currComboTimes >= 2  % Do not display 'x1', that's stupid ;P
%                 TextPsych.String{3} = ['X ', num2str(currComboTimes)];   % Number of 'Super' combos achieved
%             end
        else
            currComboTimes = 0;  % If a combo is interrupted, restart combo counter
        end


        % Update Statistics
        nTotalClearedLines = nTotalClearedLines + nLines;

        % Get how many lines to clear to get to the next level, the word
        % 'Level' in this function actually means 'line'. I used to use the
        % word 'level' and 'line' interchangeably sometimes. Sorry for the
        % inconvenience :P
        [nMoreLinesBeforeNextLevel currGameLevel]= tetrisGetRemainedLevels(nTotalClearedLines,nLinesForEachLevelCurr, theLevelPlayerStartsFrom);
        
        [NormalFallSpeed KeySampRate HorizontalMoveCoolDownTime RotateCoolDownTime] = getFallingSpeed(currGameLevel);
        curFallSpeed = NormalFallSpeed;
        %        Screen('DrawText', w, num2str(nTotalClearedLines), 12, 12, [255 255 255]);
        %        Screen('DrawText', w, num2str(nMoreLinesBeforeNextLevel),
        %        12, 100, [255 255 255])
        currPsychStatus = 1;    % Added July 8, 2009   To disable the 'Combo' scoring formula
        [currScore lastScoreIncrement] = tetrisUpdateScore(currScore, lastScoreIncrement, currPsychStatus, currPsychLines, currComboTimes);
    else
        [currScore lastScoreIncrement] = tetrisUpdateScore(currScore);
    end
    deleteLines(filledLines);

    %% Differential redraw to optimize the redraw process
    %      if ~isempty(filledLines)
    %         redrawPlayfieldPF(1,max(filledLines)); % Repaint a part of the
    %         scene
    %         %hideLines(filledLines);
    %     else
    %         redrawPlayfieldPF(curTetradY-REFPOINT_ON_BLOCK.Y+1, min(curTetradY-REFPOINT_ON_BLOCK.Y+4, PF_HEIGHT_INT));
    %     end
    drawPFOutline;
    redrawPlayfieldPF();
% % %     if TextPsych.LifeTime>0;    % Which means this Text Structure has been revived
% % %         Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]); % Clear the Text by covering it.
% % %         %         TextPsych = showText(TextPsych, TextPsych.Position);    % Here, do not use lastPsychTextPos 'cause this is probably the
% % %         % first time to show the Text
% % %         [TextPsych] = showText(TextPsych);
% % %     elseif ~isempty(nonzeros(TextPsych.Color - TextPsych.BkgColor))
% % %         TextPsych.Color = TextPsych.BkgColor;
% % %         Screen('FillRect', w, [0 0 0], [0 0 PF_TOPXY_EXT(1)-1, winRect(4)]);
% % %     end
    tetrisScoreBoard
    if isempty(currGameLevel)    % If all levels have been passed
        updateScoreBoard(nextTetradMatrix, nextColor, currScore,nTotalClearedLines,nMoreLinesBeforeNextLevel, nMaxGameLevels);
%         Screen('CopyWindow', w, woff_whole);
%         Screen('Flip',w, [], 0);

        tetrisGameEnding(1, currScore);
        esc_flag = 0;   % Return to main menu
        break;
    else
        updateScoreBoard(nextTetradMatrix, nextColor, currScore,nTotalClearedLines,nMoreLinesBeforeNextLevel, currGameLevel);
    end
    %     Screen('DrawText', w, num2str(nTotalClearedLines), 12, 12, [255 255 255]);
    %     Screen('DrawText', w, num2str(nMoreLinesBeforeNextLevel), 12, 100, [255 255 255]);
    
    % ELAPSED TIME
    elapsedTime = displayElapsedTime(timerLimitStart, GetSecs);
    if elapsedTime > TimeLimit && TimeLimit~=0; Screen('Flip', w, [], 1); tetrisGameEnding(2, currScore); esc_flag = 0; break; end   
    Screen('Flip',w,[],0);
    
    WaitSecs(0.2);
    % Check whether downKey was pressed right before the next tetrad
    % appears
    [keyDown sec keyCode] = KbCheck;
    if keyCode(DownKey)
        lastDownKeyPressed = 1;
    else
        lastDownKeyPressed = 0;
    end
    %
    %     downKeyEverReleased = 0;
    %
%     if esc_flag||restart_flag
%         restart_flag = 0;
%         break;
%     end
end

PSYCHDATA.SCORE = currScore;
PSYCHDATA.ELAPSEDTIME = elapsedTime;
if strcmpi(TETRIS_CONTEXT, 'psych')
    tetrisSaveData();
end

Screen('Flip',w, [], 0);    % To clear the screen for the display of main menu
if esc_flag     % Now in Level 1 Cycle
    break;
end
end
ListenChar(1);
Screen('CloseAll');
tetrisUnloader(oldScreenSettings);
clear all;
